#pragma once
#include "Persistence.h"
#include "LittleFS.h"
#include <sstream>
#include <iterator>
#include <cstddef>

namespace userdb
{
    enum USERATTRIBUTES
    {
        LINE_ID,
        USER_ID,
        FIRST_NAME,
        LAST_NAME,
        RFID_UID,
        USER_PIN,
        ENABLED
    };
    struct User
    {
        // User(unsigned long new_uid): uid(new_uid){}
        unsigned long uid;
        unsigned int line;
        mutable String first_name;
        mutable String last_name;
        mutable String rfid_uid;
        mutable String user_pin;
        mutable bool enabled = true;
        bool operator<(const User &o) const { return uid < o.uid; }
        bool operator==(const User &o) const { return uid == o.uid; }
        String toString()
        {
            return "ID: " + String(uid) + " Name: " + last_name + " " + first_name + " Enabled: " + (enabled ? "Yes" : "No");
        }
    };

    class UserDb
    {
    private:
    public:
        UserDb();
        void print_to_serial();
        void PrintAll();
        // User find_user();
        User user_by_pin(String);
        User user_by_rfid(String);
        User user_by_uid(unsigned long);
        static User read_csv_line(String instring, int line = -1)
        {
            int locations[5];
            int location_index = 0;
            int str_length = instring.length() - 1;
            for (int i = 0; i < str_length; i++)
            {
                if (instring.charAt(i) == ',')
                {
                    locations[location_index] = i;
                    location_index++;
                }
            }
            User res;
            res.uid = instring.substring(0, locations[0]).toInt();
            res.first_name = instring.substring(locations[0] + 1, locations[1]);
            res.last_name = instring.substring(locations[1] + 1, locations[2]);
            res.rfid_uid = instring.substring(locations[2] + 1, locations[3]);
            res.user_pin = instring.substring(locations[3] + 1, locations[4]);
            res.enabled = instring.charAt(locations[4] + 1) == '1' ? true : false;
            res.line = line;
            return res;
        }

        struct Iterator
        {
            using iterator_category = std::input_iterator_tag;
            using pointer = void;
            using value_type = User;
            using reference = User;
            using difference_type = std::ptrdiff_t;

            Iterator(File data) : db_file(data)
            {
                if (!data)
                {
                    Serial.println("Failed to open userdb while creating an iterator");
                }
                else
                {
                    next();
                }
            }
            reference operator*() const { return current; }
            Iterator &operator++()
            {
                next();
                return *this;
            }
            Iterator operator++(int)
            {
                next();
                return *this;
            }
            ~Iterator()
            {
                db_file.close();
            }
            bool has_next()
            {
                return db_file.available();
            }
            void next()
            {
                if (has_next())
                {
                    String temp = db_file.readStringUntil('\n');
                    current = read_csv_line(temp, line);
                    line++;
                }
                else
                {
                    db_file.close();
                }
            };

        private:
            File db_file;
            User current;
            unsigned int line = 0;
        };
        Iterator begin() { return Iterator(LittleFS.open("userdb.csv", "r")); }
    };
}